@@ -1,8 +1,8 @@
 use proc_macro2::{Ident, Span};
 use quote::ToTokens;
+use syn::Token;
 use syn::{Error, Receiver};
 use syn::{ForeignItemType, LitStr};
-use syn::{Token, Type};
 
 pub(crate) enum ParseError {
     /// `extern {}`
@@ -26,34 +26,12 @@ pub(crate) enum ParseError {
     /// Declared a type that we already support.
     /// Example: `type u32`
     DeclaredBuiltInType { ty: ForeignItemType },
-    /// By default, we do not allow you to pass an owned opaque type from Swift -> Rust.
-    ///
-    /// If we allowed this, Rust would drop the type, then the Swift class instance for the type
-    /// would try to drop in again, leading to double free bugs.
-    ///
-    /// One can use the `#[swift_bridge(owned_arg = "enabled")] type MyType` attribute in order to
-    /// support
-    ///
-    /// # Examples
-    ///
-    /// ```no_run
-    /// // Not allowed
-    /// extern "Rust" {
-    ///     type Foo;
-    ///     fn foo (arg: Foo);
-    /// //               --- Owned foreign type arguments are not allowed.
-    /// }
-    /// ```
-    OwnedForeignTypeArgNotAllowed { ty: Type },
     /// A bridge module struct with one or more fields must have a
     /// `#\[swift_bridge(swift_repr ="...")\[\]` attribute so that we know whether to create a
     /// `struct` or `class` on the Swift side.
     StructMissingSwiftRepr { struct_ident: Ident },
     /// Only "class" and "struct" can be used as swift_repr.
-    StructInvalidSwiftRepr {
-        struct_ident: Ident,
-        swift_repr_attr_value: LitStr,
-    },
+    StructInvalidSwiftRepr { swift_repr_attr_value: LitStr },
     /// There is no reason to use `swift_repr = "class"` on an empty struct.
     /// It's extra overhead with no advantages.
     EmptyStructHasSwiftReprClass {
@@ -106,20 +84,6 @@ self: &mut SomeType
                 );
                 Error::new_spanned(ty, message)
             }
-            ParseError::OwnedForeignTypeArgNotAllowed { ty } => {
-                let message = format!(
-                    r#"By default, foreign type arguments must be taken by reference.
-
-To allow an opaque type to be an owned argument:
-
-```
-// Valid values are "disabled", "enabled" and "enabled_unchecked" 
-#[swift_bridge(owned_arg = "enabled")]
-type SomeType;
-```"#
-                );
-                Error::new_spanned(ty, message)
-            }
             ParseError::StructMissingSwiftRepr { struct_ident } => {
                 let message = format!(
                     r#"Shared structs with one or more fields must specify their swift
@@ -139,7 +103,6 @@ TODO: Link to documntation on how to decide on the swift representation.
                 Error::new_spanned(struct_ident, message)
             }
             ParseError::StructInvalidSwiftRepr {
-                struct_ident: _,
                 swift_repr_attr_value,
             } => {
                 let message = r#"Invalid value. Must be either "class" or "struct"#;
